# -*- coding: utf-8 -*-

from django.contrib.auth.models import User
from django.core.wsgi import get_wsgi_application
from webtest import TestApp
import pytest

import eopayment
from combo.apps.lingo.models import Regie, BasketItem, Transaction
from decimal import Decimal

pytestmark = pytest.mark.django_db

@pytest.fixture
def admin_user():
    try:
        user = User.objects.get(username='admin')
    except User.DoesNotExist:
        user = User.objects.create_superuser('admin', email=None, password='admin')
    return user

def login(app, username='admin', password='admin'):
    login_page = app.get('/login/')
    login_form = login_page.forms[0]
    login_form['username'] = username
    login_form['password'] = password
    resp = login_form.submit()
    assert resp.status_int == 302
    return app

def test_access(app, admin_user):
    app = login(app)
    resp = app.get('/manage/', status=200)
    assert '/manage/lingo/' in resp.body

def test_add_regie(app, admin_user):
    Regie.objects.all().delete()
    app = login(app)
    resp = app.get('/manage/lingo/regies/', status=200)
    resp = resp.click('New')
    resp.forms[0]['label'] = 'Test'
    resp.forms[0]['slug'] = 'test'
    resp.forms[0]['description'] = 'description'
    resp.forms[0]['service'] = 'dummy'
    assert resp.form['is_default'].checked is True
    resp = resp.forms[0].submit()
    assert resp.location == 'http://localhost:80/manage/lingo/regies/'
    assert Regie.objects.count() == 1
    regie = Regie.objects.all()[0]
    assert regie.label == 'Test'
    assert regie.is_default is True

def test_edit_regie(app, admin_user):
    test_add_regie(app, admin_user)
    app = login(app)
    resp = app.get('/manage/lingo/regies/', status=200)
    resp = resp.click('Test')
    resp.forms[0]['description'] = 'other description'
    resp = resp.forms[0].submit()
    assert resp.location == 'http://localhost:80/manage/lingo/regies/'
    assert Regie.objects.count() == 1
    regie = Regie.objects.all()[0]
    assert regie.description == 'other description'

def test_delete_regie(app, admin_user):
    test_add_regie(app, admin_user)
    app = login(app)
    resp = app.get('/manage/lingo/regies/', status=200)
    resp = resp.click('Test')
    resp = resp.click('Delete')
    assert 'Are you sure you want to delete this?' in resp.body
    resp = resp.forms[0].submit()
    assert resp.location == 'http://localhost:80/manage/lingo/regies/'
    assert Regie.objects.count() == 0

def test_add_second_regie(app, admin_user):
    test_add_regie(app, admin_user)
    regie = Regie.objects.all()[0]

    app = login(app)
    resp = app.get('/manage/lingo/regies/', status=200)
    resp = resp.click('New')
    resp.forms[0]['label'] = 'Test2'
    resp.forms[0]['slug'] = 'test2'
    resp.forms[0]['description'] = 'description'
    resp.forms[0]['service'] = 'dummy'
    assert resp.form['is_default'].checked is False
    resp = resp.forms[0].submit()
    assert resp.location == 'http://localhost:80/manage/lingo/regies/'

    assert Regie.objects.count() == 2
    assert Regie.objects.get(id=regie.id).is_default is True
    assert Regie.objects.exclude(id=regie.id)[0].is_default is False

def test_download_transaction(app, admin_user):
    test_add_regie(app, admin_user)
    regie = Regie.objects.filter(slug='test')[0]
    user = User.objects.create_user('dimebag', 'dime@bag.pan', 'pwd')
    user.last_name = u'Darëll'
    user.first_name = 'Dimebag'
    user.save()

    b_item = BasketItem.objects.create(user=user, regie=regie, subject='it\'s a subject',
                                       source_url='http://example.net', amount=18.5)
    trans1 = Transaction.objects.create(regie=regie, remote_items='remote items lol', order_id='1', user=user,
                                        bank_transaction_id='567', status=eopayment.PAID)
    trans2 = Transaction.objects.create(regie=regie, remote_items='remote items omg', order_id='2', user=user,
                                        bank_transaction_id='136', status=eopayment.PAID)
    trans1.items = [b_item,]
    trans1.save()
    trans2.items = [b_item,]
    trans2.save()
    app = login(app)
    resp = app.get('/manage/lingo/transactions/download-csv/', status=200)
    content = [i for i in [item.split(',') for item in resp.content.split('\r\n')]]
    for row in content[:-1]:
        if row[0] == '2':
            assert row[1] == trans2.bank_transaction_id
            assert unicode(row[3], 'utf-8') == '%s %s' % (user.first_name, user.last_name)
            assert Decimal(row[4]) == Decimal(trans2.amount)
            assert row[5] == b_item.subject
            assert Decimal(row[6]) == b_item.amount
        else:
            assert row[1] == trans1.bank_transaction_id
            assert unicode(row[3], 'utf-8') == '%s %s' % (user.first_name, user.last_name)
            assert Decimal(row[4]) == Decimal(trans1.amount)
            assert row[5] == b_item.subject
            assert Decimal(row[6]) == b_item.amount
